<?php

namespace Litecms\Profile\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Profile\Http\Requests\ProfileRequest;
use Litecms\Profile\Interfaces\ProfileRepositoryInterface;
use Litecms\Profile\Models\Profile;

/**
 * Resource controller class for profile.
 */
class ProfileResourceController extends BaseController
{

    /**
     * Initialize profile resource controller.
     *
     * @param type ProfileRepositoryInterface $profile
     *
     * @return null
     */
    public function __construct(ProfileRepositoryInterface $profile)
    {
        parent::__construct();
        $this->repository = $profile;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Profile\Repositories\Criteria\ProfileResourceCriteria::class);
    }

    /**
     * Display a list of profile.
     *
     * @return Response
     */
    public function index(ProfileRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Profile\Repositories\Presenter\ProfilePresenter::class)
                ->$function();
        }

        $profiles = $this->repository->paginate();

        return $this->response->title(trans('profile::profile.names'))
            ->view('profile::profile.index', true)
            ->data(compact('profiles'))
            ->output();
    }

    /**
     * Display profile.
     *
     * @param Request $request
     * @param Model   $profile
     *
     * @return Response
     */
    public function show(ProfileRequest $request, Profile $profile)
    {

        if ($profile->exists) {
            $view = 'profile::profile.show';
        } else {
            $view = 'profile::profile.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('profile::profile.name'))
            ->data(compact('profile'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new profile.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ProfileRequest $request)
    {

        $profile = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('profile::profile.name')) 
            ->view('profile::profile.create', true) 
            ->data(compact('profile'))
            ->output();
    }

    /**
     * Create new profile.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ProfileRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $profile                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('profile::profile.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('profile/profile/' . $profile->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/profile/profile'))
                ->redirect();
        }

    }

    /**
     * Show profile for editing.
     *
     * @param Request $request
     * @param Model   $profile
     *
     * @return Response
     */
    public function edit(ProfileRequest $request, Profile $profile)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('profile::profile.name'))
            ->view('profile::profile.edit', true)
            ->data(compact('profile'))
            ->output();
    }

    /**
     * Update the profile.
     *
     * @param Request $request
     * @param Model   $profile
     *
     * @return Response
     */
    public function update(ProfileRequest $request, Profile $profile)
    {
        try {
            $attributes = $request->all();

            $profile->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('profile::profile.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('profile/profile/' . $profile->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('profile/profile/' . $profile->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the profile.
     *
     * @param Model   $profile
     *
     * @return Response
     */
    public function destroy(ProfileRequest $request, Profile $profile)
    {
        try {

            $profile->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('profile::profile.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('profile/profile/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('profile/profile/' . $profile->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple profile.
     *
     * @param Model   $profile
     *
     * @return Response
     */
    public function delete(ProfileRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('profile::profile.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('profile/profile'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/profile/profile'))
                ->redirect();
        }

    }

    /**
     * Restore deleted profiles.
     *
     * @param Model   $profile
     *
     * @return Response
     */
    public function restore(ProfileRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('profile::profile.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/profile/profile'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/profile/profile/'))
                ->redirect();
        }

    }

}
